unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, db, FileUtil, ZConnection, ZDataset,
  Forms, Controls, Graphics, Dialogs, StdCtrls, DBGrids;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Connection: TZConnection;
    Datasource: TDatasource;
    DBGrid: TDBGrid;
    Query: TZQuery;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    function MainConnect: Boolean;                          //Funktioniert die generelle Verbindung?
    function TestDbConnect: Boolean;                        //Kann eine Verbindung zur TestDB hergestellet werden (falls nicht vorhanden, wird die Datenbank und User angelegt)
    function CreateTab1: Boolean;                           //Erstelle eine Tabelle, die für den Test dient
    procedure WertAdd(aI, aID: Integer;                     //Füllen einer Tabellespalte
      const Spalte: String);
  public
    const SQLPfad =                                         //Der Pfad für die Postgres-Programmdateien
      'C:\Program Files (x86)\PostgreSQL\9.3\bin\';
    const PostgresPassword = '******';                      //Passwort entsprechend bei Installation von PostgreSQL vergeben eintragen
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  if not MainConnect then exit;
  if not TestDbConnect then exit;
  if not CreateTab1 then exit;
  DataSource.DataSet:=Query;
  DBGrid.DataSource:=DataSource;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  i: Integer;
  ColName: String;
begin
  for i:=0 to 9 do begin
    ColName:='spalte'+IntToStr(i);
    WertAdd(Random(10), Random(10), ColName);
  end;
  Query.SQL.Text:='SELECT * FROM tab1 ORDER BY id ASC;';
  Query.Open;
end;

function TForm1.MainConnect: Boolean;
begin
  Result:=False;
  Connection.Database:='postgres';                          //Diese Datenbank wird immer bei der Installation von Postgres erstellt
  Connection.HostName:='localhost';
  Connection.LibraryLocation:=SQLPfad+'libpq.dll';
  Connection.Password:=PostgresPassword;                    //Passwort bei Installation von PostgreSQL vergeben
  Connection.Port:=5432;
  Connection.Protocol:='postgresql-9';
  Connection.User:='postgres';

  Query.Connection:=Connection;
  try                                                       //Versuche eine Verbindung herzustellen
    Connection.Connected:=true;                             //Pfad von Postgres muss in Windows-System-Pfad eingefügt werden
                                                            //Windows-Explorer öffnen
                                                            //Rechtsklick auf "Computer"
                                                            //dann Eigenschaften -> Erweiterte Systemeinstellungen
                                                            // -> Reiter "Erweitert" -> Umgebungsvariablen -> SystemVariablen
                                                            // -> Path -> Verzeichnis von Postgres/bin eintragen! Neustart Computer und schon gehts
  except
    on E: Exception do begin
      exit;
    end;
  end;
  Result:=True;
end;

function TForm1.TestDbConnect: Boolean;
var
  DBCount:Int32;
begin
  Result:=False;
  Query.SQL.Text:='SELECT COUNT(*) FROM PG_CATALOG.PG_DATABASE WHERE DATNAME = '+#39+'testdb'+#39+';';
  Query.Open;

  if Query.FieldCount>0 then                                //wurde eine Antwort erhalten, wenn ja, dann
    DBCount:=Query.Fields[0].AsInteger                      //DBCount=0 keine / DBCount=1 wenn Datenbank existiert
  else begin                                                //keine Antwort auf SQL-Befehl
    exit;
  end;
  Query.Close;

  if DBCount=0 then                                         //Die Datenbank existiert noch nicht, daher erstellen!
    try
      Connection.ExecuteDirect('CREATE USER testdb WITH PASSWORD '+#39+'test'+#39+';');  //testdb als User generieren
      Connection.ExecuteDirect('ALTER USER testdb VALID UNTIL '+#39+'infinity'+#39+';'); //Passwort erlischt niemals
      Connection.ExecuteDirect('ALTER USER testdb CREATEDB NOCREATEROLE REPLICATION;');  //User ema darf keine neuen User generieren
      Connection.ExecuteDirect('CREATE DATABASE testdb;');                               //Datenbank TestDB anlegen
      Connection.ExecuteDirect('GRANT ALL PRIVILEGES ON DATABASE testdb TO testdb;');
    except
      on E: Exception do begin
        exit;
      end;
    end;
  Connection.Disconnect;

  Connection.Database:='testdb';
  Connection.HostName:='localhost';
  Connection.LibraryLocation:=SQLPfad+'libpq.dll';
  Connection.Password:='test';
  Connection.Port:=5432;
  Connection.Protocol:='postgresql-9';
  Connection.User:='testdb';

  Query.Connection:=Connection;

  try                                                       //Versuche eine Verbindung herzustellen
    Connection.Connected:=true;
  except
    on E: Exception do begin
      exit;
    end;
  end;
  Result:=True;
end;

function TForm1.CreateTab1: Boolean;
begin
  Result:=False;

  try
    Query.SQL.Text:='SELECT ''testdb'' FROM information_schema.tables WHERE table_name=''tab1'';';
    Query.Open;

    if Query.RecordCount > 0 then begin                     //Tabelle "Tab1" existiert schon, da ich sie leer haben will, lösche ich sie einfach und erstelle sie dann neu
      Query.Close;
      Query.SQL.Text:='DROP TABLE tab1;';
      Query.ExecSQL;
    end else
      Query.Close;

    Query.SQL.Text:='CREATE TABLE tab1(id INTEGER PRIMARY KEY);';
    Query.ExecSQL;
  except
    on E: Exception do begin
      exit;
    end;
  end;
  Result:=True;
end;

procedure TForm1.WertAdd(aI, aID: Integer;
  const Spalte: String);
var
  ColDa: Boolean;                                           //Ist die Spalte in der Tabelle vorhanden?

  procedure ColVorhanden;                                   //Ist die Spalte in der Tabelle vorhanden?
  begin
    ColDa:=False;
    Query.SQL.Text:='SELECT column_name FROM information_schema.COLUMNS WHERE table_name = ''tab1'';';
    Query.Open;

    if Query.RecordCount > 0 then begin

      Query.Locate('column_name', Spalte, []);
      if Query.FieldByName('column_name').AsString = Spalte then ColDa:=True;

    end;
    Query.Close;
  end;
  function SpalteAnlegen: Boolean;                          //Legt die Spalte in der Tabelle an
  var
    i: Int32;
  begin
    Connection.ExecuteDirect('ALTER TABLE tab1 ADD COLUMN '+Spalte+' INTEGER;');


    Connection.Disconnect;                                  //Warum muss ich hier Disconnecten und wieder Connecten???
    Connection.Connect;


    i:=0;
    while not ColDa and (i<100) do begin
      ColVorhanden;
      if not ColDa then sleep(20);
    end;

    if ColDa
      then Result:=True
      else Result:=False;
  end;
  function IstIdDa: Boolean;                                //Ist die ID in der Tabelle schon vorhanden?!
  begin
    Query.SQL.Text:='SELECT * FROM tab1 WHERE id = :id;';
    Query.ParamByName('id').AsInteger:=aID;
    Query.Open;
    if Query.RecNo = 0
      then Result:=False
      else Result:=True;
    Query.Close;
  end;
  procedure IDUndWertEinfuegen;                             //ID und Wert in Tabelle einfügen
  begin
    Query.SQL.Text:='INSERT INTO tab1 (id, '+Spalte+') VALUES (:id, :wert);';
    Query.ParamByName('id').AsInteger:=aID;
    Query.ParamByName('wert').AsInteger:=aI;
    Query.ExecSQL;
  end;
  procedure WertEinfuegen;                                  //Wert in Tabelle bei Datúm einfügen
  begin
    Query.SQL.Text:='UPDATE tab1 SET '+Spalte+' = :wert WHERE id = :id;';
    Query.ParamByName('id').AsInteger:=aID;
    Query.ParamByName('wert').AsInteger:=aI;
    Query.ExecSQL;
  end;

begin
  ColVorhanden;                                             //Ist die Spalte in der Tabelle vorhanden?

  if not ColDa then
    if not SpalteAnlegen then Exit;

  if not IstIdDa
    then
      IDUndWertEinfuegen
    else
      WertEinfuegen;                                        //Wert in BasisTabelle einfügen
end;

end.

